/**
 * 
 */
package com.wjholden.kurosuke;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.UnknownHostException;

import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings.Secure;
import android.util.Log;

/**
 * @author John
 *
 */
public class NetworkLogic extends Thread implements KurosukeConstants {

	public static Handler handler;
	

	
	private boolean closing = false;
	
	private MulticastSocket socket;
	private InetAddress group;
	
	/**
	 * 
	 */
	public NetworkLogic()
	{
		Log.d(TAG_NETWORK, "Created new NetworkLogic object.");
		handler = new Handler()
		{
			public void handleMessage(Message msg)
			{
				switch (msg.what)
				{
				case SEND_KEYS:
					Bundle b = msg.getData();
					sendKeys(b, msg.arg1);
					break;
				case CLOSE:
					closing = true;
					break;
				default:
					Log.d(TAG_NETWORK, "Handler received unrecognized command.");
				}
			}
		};
	}
	
	/**
	 * Most of this networking code comes directly from 
	 * http://download.oracle.com/javase/7/docs/api/java/net/MulticastSocket.html
	 * 5683 = "love" on your telephone keypad
	 */
	public void run() {
		Log.i(TAG_NETWORK, "Started new thread for NetworkLogic.");
		
		try {
			group = InetAddress.getByName(IP_GROUP);
		} catch (UnknownHostException e1) {
			Log.e(TAG_NETWORK, e1.toString());
		}
		
		try {
			socket = new MulticastSocket(PORT);
			socket.joinGroup(group);
		} catch (IOException e1) {
			Log.e(TAG_NETWORK, e1.toString());
			return; // crash if there's a problem connecting
		}
		
		while (!closing)
		{
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				Log.e(TAG_NETWORK, e.toString());
			}
		}
		
		try {
			socket.leaveGroup(group);
		} catch (IOException e) {
			Log.e(TAG_NETWORK, e.toString());
		}
		
		Log.i(TAG_NETWORK, "NetworkLogic thread closed normally.");
	}
	
	/**
	 * 
	 * @param b - must contain integer keys: up, down, left, right, a, b
	 * @param playerId [1,4] indicator of which player it is
	 */
	private void sendKeys(Bundle b, int playerId)
	{
		DatagramPacket packet;
		StringBuilder sb;
		String s;
		
		/*
		 * Future version may utilize holdUntilReleased; 0.1 does not, therefore it is removed.
		sb = new StringBuilder("<?xml version='1.0' encoding='ISO-8859-1' standalone='yes'?>\n<input>");
		sb.append("\t<keyPress playerId='" + playerId + "'>\n");
		sb.append("\t\t<up holdUntilReleased='true'>"    + b.getInt("up")    + "</up>\n");
		sb.append("\t\t<down holdUntilReleased='true'>"  + b.getInt("down")  + "</down>\n");
		sb.append("\t\t<left holdUntilReleased='true'>"  + b.getInt("left")  + "</left>\n");
		sb.append("\t\t<right holdUntilReleased='true'>" + b.getInt("right") + "</right>\n");
		sb.append("\t\t<btnA holdUntilReleased='false'>" + b.getInt("a")     + "</btnA>\n");
		sb.append("\t\t<btnB holdUntilReleased='false'>" + b.getInt("b")     + "</btnB>\n");
		sb.append("\t</keyPress>\n</input>\n");
		*/
		
		sb = new StringBuilder("<?xml version='1.0' encoding='ISO-8859-1' standalone='yes'?>\n<input>");
		sb.append("\t<keyPress playerId='" + playerId + "' androidId='" + Secure.ANDROID_ID + "'>\n");
		sb.append("\t\t<up>"    + b.getInt("up")    + "</up>\n");
		sb.append("\t\t<down>"  + b.getInt("down")  + "</down>\n");
		sb.append("\t\t<left>"  + b.getInt("left")  + "</left>\n");
		sb.append("\t\t<right>" + b.getInt("right") + "</right>\n");
		sb.append("\t\t<btnA>"  + b.getInt("a")     + "</btnA>\n");
		sb.append("\t\t<btnB>"  + b.getInt("b")     + "</btnB>\n");
		sb.append("\t</keyPress>\n</input>"); // bothers me that there's no newline, but the spec says no.
		
		s = sb.toString();
		
		packet = new DatagramPacket(s.getBytes(), s.length(), group, PORT);
		try {
			socket.send(packet);
			if (DEBUG) Log.d(TAG_NETWORK, s);
		} catch (IOException e) {
			Log.e(TAG_NETWORK, e.toString());
		}
	}
}
